var History = {};

/**

 * HistoryManager
 * 
 * Observes back/forward button usage and saves states
 * for registered modules into the hash. This allows to
 * bookmark specific states for an application.
 * 
 * @version		1.0rc2
 * 
 * @see			Events, Options
 * 
 * @license		MIT License
 * @author		Harald Kirschner <mail [at] digitarald.de>
 * @copyright	2007 Author
 */
History.System = new Class ({

  Implements: [Options, Events],

	/**
	 * Default options - Can be overridden with setOptions
	 * 
	 * observeDelay: Duration for checking the state, default 100ms
	 * iframeSrc: Scr for IE6/7 iframe, must exist on server!
	 * onStart: Fires on start
	 * onRegister: Fires on register
	 * onUnregister: Fires on unregister
	 * onUpdate: Fires when state changes from ...
	 * onPlaceChange: ... module changes
	 */
	options: {
		observeDelay: 100,
		iframeSrc: '/history/history.html'
	},

	/**
	 * Constructor - Class.initialize
	 * 
	 * Options:
	 *  - observeDelay: duration in ms, default 100 - BackBuddy observe the hash for changes periodical
	 *  - iframeSrc: string, default 'blank.html' - File for the iframe (IE6/7), must exist on the server!
	 *  - Events: onStart, onStart, onPlaceChange
	 * 
	 * @return	this
	 * 
	 * @param	{Object} options
	 */
	initialize: function(options) {
		this.setOptions(options);
		this.state = null;
		return this;
	},

	/**
	 * Start - Check hash and start observer
	 * 
	 * Call start after registering ALL modules. This start the observer,
	 * reads the state from the hash and calls onMatch for effected modules.
	 * 
	 * @return	this
	 * 
	 */
	start: function() {
		this.observe.periodical(this.options.observeDelay, this);
		this.started = true;
		this.observe();
		
		this.setState(this.state, true);
		window.fireEvent('historystart', [this.state]);

		return this;
	},


	observe: function() {
		
		if (this.timeout) {
			return;
		}
		
		var state = this.getState();
		
		// if the state hasn't changed since the last observations then do nothing but return
		if (this.state == state){
			return;
		} 
		
		// if IE < 8 and state is set
		if (((Browser.Engine.trident &&(!document.querySelectorAll))) && (this.state !== null)) {
			this.setState(state, false);
		}
		else {
			this.state = state;
		}

		window.fireEvent('placechange', [state]);
	},

	observeTimeout: function() {
		if (this.timeout) this.timeout = $clear(this.timeout);
		else this.timeout = this.observeTimeout.delay(200, this);
	},

	getHash: function() {
		var href = top.location.href;
		var pos = href.indexOf('#') + 1;
		return (pos) ? href.substr(pos) : '';
	},

	getState: function() {
		var state = this.getHash();
		if (this.iframe) {
			var doc = this.iframe.contentWindow.document;
			if (doc && doc.body.id == 'state') {
				var istate = doc.body.innerText;
				if (this.state == state) return istate;
				this.istateOld = true;
			} else return this.istate;
		}
		return state;
	},

	setState: function(state, initial) {
		state = $pick(state, '');
		top.location.hash = state || '#';
		
		// if IE < 8 and state is set or initial
		if (Browser.Engine.trident && (!document.querySelectorAll) && (initial || this.istateOld)) {
			if (!this.iframe) {
				this.iframe = new Element('iframe', {
					src: this.options.iframeSrc,
					style: 'visibility: hidden;height:1px;'
				}).injectInside(document.body);
				this.istate = this.state;
			}
			try {
				var doc = this.iframe.contentWindow.document;
				doc.open();
				doc.write('<html><body id="state">' + state + '</body></html>');
				doc.close();
				this.istateOld = false;
			} catch(e) {};
		}
		this.state = state;
	}

});


History.Manager = new Class ({
	
	system: null,
	observers: [],
	
	initialize: function(system) {
	
		this.system = system.start();
		
		window.addEvent('placechange', function(state) {
			this.observers.each(function(observer){
				this.notify(observer, state);
			}, this);
		}.bind(this));
	},
	
	addListener: function(callback){
		var observer = {'callback':callback};
		// Add observer to list of observers
		this.observers.push(observer);
		// On add notify the observer for initial state
		var state = this.getState();
		this.notify(observer, state);
	},
	
	notify: function(observer, state){
		observer.callback(state);
	},
	
	getState: function(){
		return this.system.getState();
	}

});

/**
 * If you're using the History API means 
 * a) your browser supports it 
 * b) you're not in testing mode
 * @returns {Boolean}
 */
History.usingHistoryApi = function()
{
	return !History.forceOldMode && History.supportsHistoryApi();
};

/**
 * Detect History API support (HTML5)
 * @see http://diveintohtml5.org/detect.html#history
 * @returns {Boolean}
 */
History.supportsHistoryApi = function()
{
	return !!(window.history && history.pushState);
};

/**
 * Singleton pattern
 */
History.instance = function(){
	
	if (!History._instance){
		History._instance = new History.Manager(new History.System);
	}
	return History._instance;	
};

// Override this to force Old mode
History.forceOldMode = false;
